# -*- coding: utf-8 -*-

"""
    Covenant Add-on

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

import unicodedata
from ptw.libraries.utils import convert


def get(title):
    if title is None:
        return
    title = re.sub('&#(\d+);', '', title)
    title = re.sub('(&#[0-9]+)([^;^0-9]+)', '\\1;\\2', title)
    title = title.replace('&quot;', '\"').replace('&amp;', '&')
    title = re.sub('', '\n|([[].+?[]])|([(].+?[)])|\s(vs|v[.])\s|([:;\-–",)|\s', title).lower()
    return title


def geturl(title):
    if title is None:
        return
    title = title.lower()
    title = title.translate(None, ':*?"\'\.<>|&!,')
    title = title.replace('/', '-')
    title = title.replace(' ', '-')
    title = title.replace('--', '-')
    return title


def get_simple(title):
    if title is None:
        return
    title = title.lower()
    title = re.sub('(\d{4})', '', title)
    title = re.sub('&#(\d+);', '', title)
    title = re.sub('(&#[0-9]+)([^;^0-9]+)', '\\1;\\2', title)
    title = title.replace('&quot;', '\"').replace('&amp;', '&')
    title = re.sub('\n|\(|\)|\[|\]|\{|\}|\s(vs|v[.])\s|(:|;|-|–|"|,|\'|\_|\.|\?)|\s', '', title).lower()
    return title


def getsearch(title):
    if title is None:
        return
    title = title.lower()
    title = re.sub('&#(\d+);', '', title)
    title = re.sub('(&#[0-9]+)([^;^0-9]+)', '\\1;\\2', title)
    title = title.replace('&quot;', '\"').replace('&amp;', '&')
    title = re.sub('\\\|/|!|\[|\]|–|:|;|\*|\?|"|\'|<|>|\|', '', title).lower()
    title = title.replace(".", " ").replace("  ", " ")
    return title


def query(title):
    if title is None:
        return
    title = title.replace('\'', '').rsplit(':', 1)[0].rsplit(' -', 1)[0].replace('-', ' ')
    return title


def normalize(title):
    try:
        return str(''.join(
            c for c in unicodedata.normalize('NFKD', convert(title)) if unicodedata.category(c) != 'Mn')).replace('ł',
                                                                                                                  'l')
    except:
        title = convert(title).replace('ą', 'a').replace('ę', 'e').replace('ć', 'c').replace('ź', 'z').replace('ż',
                                                                                                               'z').replace(
            'ó', 'o').replace('ł', 'l').replace('ń', 'n').replace('ś', 's')
        return title


def clean_search_query(url):
    url = url.replace('-', '+')
    url = url.replace(' ', '+')
    return url
